==> ?obj=sdir%2F1.20%2Fsdir-source.zip: sdir-1.20/usage.c <==
/*
 * USAGE.C
 *
 * Colorful, sorted and optionally rich directory enumeration
 * for Windows.
 *
 * This module contains help text to display to the user for invalid arguments
 * or upon request.
 *
 * Copyright (c) 2014 Malcolm J. Smith
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include "sdir.h"

const
CHAR strUsage1[] = "  Usage: [opts] [pathspec] ...\n"
                   "\n"
                   "   -?           Display usage\n"
                   "\n"
                   "   -cw[num]     Width of console when writing to files\n"
                   "   -p/-pn       Pause/no pause after each screen\n"
                   "   -t/-tn       Truncate/no truncate of very long file names\n"
#ifdef UNICODE
                   "   -u/-un       Unicode/no unicode output when writing to files\n"
#endif
                   "\n";

const
CHAR strUsage2[] = "\n"
                   " Options can also be in the SDIR_OPTS environment variable.\n"
                   " Processed in order, environment then arguments.\n"
                   "\n"
                   " Color coding is defined via three environment variables, processed in order:\n"
                   "   SDIR_COLOR_PREPEND, processed first\n"
                   "   SDIR_COLOR_REPLACE, if defined; otherwise, built in defaults apply\n"
                   "   SDIR_COLOR_APPEND, processed last\n"
                   "\n"
                   " Each variable contains a semicolon delimited list of rules.  Each rule takes\n"
                   " the form of:\n"
                   "\n"
                   "   [file attribute][operator][criteria],[color]\n"
                   "\n"
                   " Valid operators are:\n"
                   "   =   File attribute matches criteria\n"
                   "   !=  File attribute does not match criteria\n"
                   "   >   File attribute greater than criteria\n"
                   "   >=  File attribute greater than or equal to criteria\n"
                   "   <   File attribute less than criteria\n"
                   "   <=  File attribute less than or equal to criteria\n"
                   "   &   File attribute includes criteria\n"
                   "   !&  File attribute does not include criteria\n"
                   "\n"
                   " Valid attributes and corresponding operators are:\n";

const
CHAR strUsage3[] = "\n"
                   " The built in default set for this build is:\n";


BOOL
SdirUsage()
{

    DWORD   i;
    TCHAR   Line[80];
    TCHAR   strUsage[sizeof(strUsage2) + 1];
    LPCSTR  This;

    _stprintf_s(Line,
                sizeof(Line)/sizeof(Line[0]),
                _T("Sdir version %i.%i, compiled %hs, %hs, %hs"),
                SDIR_VER_MAJOR,
                SDIR_VER_MINOR,
                __DATE__,
#ifdef UNICODE
                "Unicode",
#else
                "ANSI",
#endif
#ifdef MINICRT
                "Minicrt"
#else
                "Msvcrt"
#endif
                );

    if (!SdirWriteString(Line)) {
        return FALSE;
    }

#ifdef _MSC_VER

    _stprintf_s(Line,
                sizeof(Line)/sizeof(Line[0]),
                _T(", C version %i.%02i"),
                _MSC_VER / 100,
                _MSC_VER % 100);

    if (!SdirWriteString(Line)) {
        return FALSE;
    }
#endif

    if (!SdirWriteString(_T("\n\n"))) {
        return FALSE;
    }

    SdirAnsiToUnicode(strUsage, sizeof(strUsage)/sizeof(strUsage[0]), strUsage1);

    if (!SdirWriteString(strUsage)) {
        return FALSE;
    }
    for (i = 0; i < SdirGetNumSdirOptions(); i++) {
        if (SdirOptions[i].Default & SDIR_OPT_ALLOW_DISPLAY) {

            _stprintf_s(Line,
                        sizeof(Line)/sizeof(Line[0]),
                        _T("   -d%s/-h%s    Display/hide %hs%hs\n"),
                        SdirOptions[i].Switch,
                        SdirOptions[i].Switch,
                        SdirOptions[i].Help,
                        (SdirOptions[i].Default&SDIR_OPT_DISPLAY)?" (displayed by default)":"");

            if (!SdirWriteString(Line)) {
                return FALSE;
            }
        }
    }

    _stprintf_s(Line,
                sizeof(Line)/sizeof(Line[0]),
                _T("\n")
                _T(" Up to %i sort criteria can be applied.  Supported sort flags are:\n"),
                sizeof(Opts->Sort)/sizeof(Opts->Sort[0]));

    if (!SdirWriteString(Line)) {
        return FALSE;
    }

    for (i = 0; i < SdirGetNumSdirOptions(); i++) {
        if (SdirOptions[i].Default & SDIR_OPT_ALLOW_SORT) {

            _stprintf_s(Line,
                        sizeof(Line)/sizeof(Line[0]),
                        _T("   -s%s/-i%s    Sort/inverse sort by %hs\n"),
                        SdirOptions[i].Switch,
                        SdirOptions[i].Switch,
                        SdirOptions[i].Help);

            if (!SdirWriteString(Line)) {
                return FALSE;
            }
        }
    }

    SdirAnsiToUnicode(strUsage, sizeof(strUsage)/sizeof(strUsage[0]), strUsage2);

    if (!SdirWriteString(strUsage)) {
        return FALSE;
    }
    for (i = 0; i < SdirGetNumSdirOptions(); i++) {
        if (SdirOptions[i].GenerateFromStringFn != NULL) {

            _stprintf_s(Line,
                        sizeof(Line)/sizeof(Line[0]),
                        _T("   %s (%hs), %hs%hs%hs\n"),
                        SdirOptions[i].Switch,
                        SdirOptions[i].Help,
                        SdirOptions[i].CompareFn?"=, !=, >, >=, <, <=":"",
                        (SdirOptions[i].CompareFn && SdirOptions[i].BitwiseCompareFn)?", ":"",
                        SdirOptions[i].BitwiseCompareFn?"&, !&":"");

            if (!SdirWriteString(Line)) {
                return FALSE;
            }
        }
    }
    SdirAnsiToUnicode(strUsage, sizeof(strUsage)/sizeof(strUsage[0]), strUsage3);
    if (!SdirWriteString(strUsage)) {
        return FALSE;
    }

    //
    //  We want to display built in rules, but for formatting's sake display
    //  one per line.  This is the kludge to make that happen.
    //

    This = SdirAttributeDefaultApplyString;

    _stprintf_s(Line,
                sizeof(Line)/sizeof(Line[0]),
                _T("   "));

    while (This != NULL) {

        //
        //  Copy characters including the semicolon, and add a newline and
        //  NULL.  We have three spaces at the beginning, so we need 6
        //  extra characters plus the size of the element itself.  It'd be
        //  nice if msvcrt didn't puke when the source string is larger
        //  than the specified size so we could just use a single
        //  stprintf_s for both cases.
        //

        for (i = 0; This[i] != '\0' && This[i] != ';'; i++) {
            Line[i + 3] = This[i];
        }
        Line[i + 3] = ';';
        Line[i + 4] = '\n';
        Line[i + 5] = '\0';

        if (!SdirWriteString(Line)) {
            return FALSE;
        }

        This += i;
        if (*This) {
            This++;
        } else {
            break;
        }
    }

    if (!SdirWriteString(_T("\n"))) {
        return FALSE;
    }
    
    return TRUE;
}

// vim:sw=4:ts=4:et:

==> ?obj=sdir%2F1.21%2Fsdir-source.zip: sdir-1.21/usage.c <==
/*
 * USAGE.C
 *
 * Colorful, sorted and optionally rich directory enumeration
 * for Windows.
 *
 * This module contains help text to display to the user for invalid arguments
 * or upon request.
 *
 * Copyright (c) 2014 Malcolm J. Smith
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include "sdir.h"

const
CHAR strUsage1[] = "  Usage: [opts] [pathspec] ...\n"
                   "\n"
                   "   -?           Display usage\n"
                   "\n"
                   "   -cw[num]     Width of console when writing to files\n"
                   "   -p/-pn       Pause/no pause after each screen\n"
                   "   -t/-tn       Truncate/no truncate of very long file names\n"
#ifdef UNICODE
                   "   -u/-un       Unicode/no unicode output when writing to files\n"
#endif
                   "\n";

const
CHAR strUsage2[] = "\n"
                   " Options can also be in the SDIR_OPTS environment variable.\n"
                   " Processed in order, environment then arguments.\n"
                   "\n"
                   " Color coding is defined via three environment variables, processed in order:\n"
                   "   SDIR_COLOR_PREPEND, processed first\n"
                   "   SDIR_COLOR_REPLACE, if defined; otherwise, built in defaults apply\n"
                   "   SDIR_COLOR_APPEND, processed last\n"
                   "\n"
                   " Each variable contains a semicolon delimited list of rules.  Each rule takes\n"
                   " the form of:\n"
                   "\n"
                   "   [file attribute][operator][criteria],[color]\n"
                   "\n"
                   " Valid operators are:\n"
                   "   =   File attribute matches criteria\n"
                   "   !=  File attribute does not match criteria\n"
                   "   >   File attribute greater than criteria\n"
                   "   >=  File attribute greater than or equal to criteria\n"
                   "   <   File attribute less than criteria\n"
                   "   <=  File attribute less than or equal to criteria\n"
                   "   &   File attribute includes criteria or wildcard string\n"
                   "   !&  File attribute does not include criteria or wildcard string\n"
                   "\n"
                   " Valid attributes and corresponding operators are:\n";

const
CHAR strUsage3[] = "\n"
                   " Multiple colors can be combined, delimited by +. Valid colors are:\n";

const
CHAR strUsage4[] = "\n"
                   " In addition to regular colors, special keywords can also be included,\n"
                   " including 'hide', 'invert', and 'bright'.  These are particularly\n"
                   " powerful when combined with the 'continue' keyword, which remembers\n"
                   " specified colors and continues processing for additional colors.\n"
                   "\n"
                   " The built in default set for this build is:\n";


BOOL
SdirUsage()
{

    DWORD   i;
    TCHAR   Line[80];
    TCHAR   strUsage[sizeof(strUsage2) + 1];
    LPCSTR  This;

    _stprintf_s(Line,
                sizeof(Line)/sizeof(Line[0]),
                _T("Sdir version %i.%i, compiled %hs, %hs, %hs"),
                SDIR_VER_MAJOR,
                SDIR_VER_MINOR,
                __DATE__,
#ifdef UNICODE
                "Unicode",
#else
                "ANSI",
#endif
#ifdef MINICRT
                "Minicrt"
#else
                "Msvcrt"
#endif
                );

    if (!SdirWriteString(Line)) {
        return FALSE;
    }

#ifdef _MSC_VER

    _stprintf_s(Line,
                sizeof(Line)/sizeof(Line[0]),
                _T(", C version %i.%02i"),
                _MSC_VER / 100,
                _MSC_VER % 100);

    if (!SdirWriteString(Line)) {
        return FALSE;
    }
#endif

    if (!SdirWriteString(_T("\n\n"))) {
        return FALSE;
    }

    SdirAnsiToUnicode(strUsage, sizeof(strUsage)/sizeof(strUsage[0]), strUsage1);

    if (!SdirWriteString(strUsage)) {
        return FALSE;
    }
    for (i = 0; i < SdirGetNumSdirOptions(); i++) {
        if (SdirOptions[i].Default & SDIR_OPT_ALLOW_DISPLAY) {

            _stprintf_s(Line,
                        sizeof(Line)/sizeof(Line[0]),
                        _T("   -d%s/-h%s    Display/hide %hs%hs\n"),
                        SdirOptions[i].Switch,
                        SdirOptions[i].Switch,
                        SdirOptions[i].Help,
                        (SdirOptions[i].Default&SDIR_OPT_DISPLAY)?" (displayed by default)":"");

            if (!SdirWriteString(Line)) {
                return FALSE;
            }
        }
    }

    _stprintf_s(Line,
                sizeof(Line)/sizeof(Line[0]),
                _T("\n")
                _T(" Up to %i sort criteria can be applied.  Supported sort flags are:\n"),
                sizeof(Opts->Sort)/sizeof(Opts->Sort[0]));

    if (!SdirWriteString(Line)) {
        return FALSE;
    }

    for (i = 0; i < SdirGetNumSdirOptions(); i++) {
        if (SdirOptions[i].Default & SDIR_OPT_ALLOW_SORT) {

            _stprintf_s(Line,
                        sizeof(Line)/sizeof(Line[0]),
                        _T("   -s%s/-i%s    Sort/inverse sort by %hs\n"),
                        SdirOptions[i].Switch,
                        SdirOptions[i].Switch,
                        SdirOptions[i].Help);

            if (!SdirWriteString(Line)) {
                return FALSE;
            }
        }
    }

    SdirAnsiToUnicode(strUsage, sizeof(strUsage)/sizeof(strUsage[0]), strUsage2);

    if (!SdirWriteString(strUsage)) {
        return FALSE;
    }

    //
    //  Display supported options and operators
    //

    for (i = 0; i < SdirGetNumSdirOptions(); i++) {
        if (SdirOptions[i].GenerateFromStringFn != NULL) {

            _stprintf_s(Line,
                        sizeof(Line)/sizeof(Line[0]),
                        _T("   %s (%hs), %hs%hs%hs\n"),
                        SdirOptions[i].Switch,
                        SdirOptions[i].Help,
                        SdirOptions[i].CompareFn?"=, !=, >, >=, <, <=":"",
                        (SdirOptions[i].CompareFn && SdirOptions[i].BitwiseCompareFn)?", ":"",
                        SdirOptions[i].BitwiseCompareFn?"&, !&":"");

            if (!SdirWriteString(Line)) {
                return FALSE;
            }
        }
    }
    SdirAnsiToUnicode(strUsage, sizeof(strUsage)/sizeof(strUsage[0]), strUsage3);
    if (!SdirWriteString(strUsage)) {
        return FALSE;
    }

    //
    //  Display supported colors
    //

    for (i = 0; i < 8; i++) {

        _stprintf_s(Line,
                    sizeof(Line)/sizeof(Line[0]),
                    _T("   %-16s"),
                    ColorString[i].String);

        SdirWriteStringWithAttribute(Line, (WORD)(i==0?7:i));

        _stprintf_s(Line,
                    sizeof(Line)/sizeof(Line[0]),
                    _T(" %-16s"),
                    ColorString[i+8].String);

        SdirWriteStringWithAttribute(Line, (WORD)(i+8));

        _stprintf_s(Line,
                    sizeof(Line)/sizeof(Line[0]),
                    _T(" bg_%-16s"),
                    ColorString[i].String);

        SdirWriteStringWithAttribute(Line, (WORD)((i<<4)+0xF));

        _stprintf_s(Line,
                    sizeof(Line)/sizeof(Line[0]),
                    _T(" bg_%-16s"),
                    ColorString[i+8].String);

        SdirWriteStringWithAttribute(Line, (WORD)((i+8)<<4));
        SdirWriteString(_T("\n"));
    }


    SdirAnsiToUnicode(strUsage, sizeof(strUsage)/sizeof(strUsage[0]), strUsage4);
    if (!SdirWriteString(strUsage)) {
        return FALSE;
    }

    //
    //  We want to display built in rules, but for formatting's sake display
    //  one per line.  This is the kludge to make that happen.
    //

    This = SdirAttributeDefaultApplyString;

    _stprintf_s(Line,
                sizeof(Line)/sizeof(Line[0]),
                _T("   "));

    while (This != NULL) {

        //
        //  Copy characters including the semicolon, and add a newline and
        //  NULL.  We have three spaces at the beginning, so we need 6
        //  extra characters plus the size of the element itself.  It'd be
        //  nice if msvcrt didn't puke when the source string is larger
        //  than the specified size so we could just use a single
        //  stprintf_s for both cases.
        //

        for (i = 0; This[i] != '\0' && This[i] != ';'; i++) {
            Line[i + 3] = This[i];
        }
        Line[i + 3] = ';';
        Line[i + 4] = '\n';
        Line[i + 5] = '\0';

        if (!SdirWriteString(Line)) {
            return FALSE;
        }

        This += i;
        if (*This) {
            This++;
        } else {
            break;
        }
    }

    if (!SdirWriteString(_T("\n"))) {
        return FALSE;
    }
    
    return TRUE;
}

// vim:sw=4:ts=4:et:
